import os
from genesys.app.services import svn_service
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL, TEMPLATE_FILES_DIR
from genesys.app.utils import config_helpers
from configparser import ConfigParser
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                LOGIN_NAME)
import bpy

def create_task_file(task, project_name, base_file_directory, base_svn_directory):
    "creates file tasks"
    acl_parser = ConfigParser()
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name.replace(' ', '_').lower(), 'conf/authz')
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)

    config_helpers.load_config(svn_authz_path, acl_parser)

    file_folder_url = os.path.dirname(blend_file_url)
    file_name = os.path.basename(base_file_directory).rsplit('.', 1)[0]
    file_map_folder_url = os.path.join(file_folder_url, 'maps', file_name)
    base_map_svn_directory = os.path.join(os.path.dirname(base_svn_directory), 'maps', file_name)

    if not svn_service.is_svn_url(file_folder_url):
        svn_service.svn_make_dirs(file_folder_url, log_message=f'created {file_folder_url}')
    if not svn_service.is_svn_url(file_map_folder_url):
        svn_service.svn_make_dirs(file_map_folder_url, log_message=f'created {file_map_folder_url}')
    if not svn_service.is_svn_url(blend_file_url):
        task_type = task['task_type']['name']
        task_id = task['id']
        entity_id = task['entity_id']
        nagato_file_data = {
            'task_type':task_type,
            'task_id':task_id, 
            'entity_id':entity_id
        }

        default_collection = bpy.data.collections.get('Collection')
        for obj in default_collection.objects:
            bpy.data.objects.remove(obj, do_unlink=True)  
        bpy.data.collections.remove(default_collection)

        main_collection = bpy.data.collections.new(file_name)
        bpy.context.scene.collection.children.link(main_collection)

        # assets = bpy.data.collections.new("assets")
        # main_collection.children.link(assets)

        bpy.data.scenes['Scene'].name = file_name
        scene = bpy.data.scenes.get(file_name)
        scene['nagato_file_data'] = nagato_file_data
        bpy.ops.wm.save_as_mainfile(filepath=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'))
        svn_service.svn_import(path=os.path.join(TEMPLATE_FILES_DIR,'blender.blend'),
                                    repo_url=blend_file_url,
                                    log_message=f'created {blend_file_url}')
                                   
    create_new_task_acl(base_svn_directory=base_svn_directory, base_map_svn_directory=base_map_svn_directory, acl_parser=acl_parser)
    config_helpers.write_config(svn_authz_path, acl_parser)

def create_new_task_acl(base_svn_directory:str, base_map_svn_directory:str, acl_parser):
    "create svn access entry for task file and set all users to no acces"
    if base_svn_directory in acl_parser:
        pass
    else:
        acl_parser[base_svn_directory] = {
            '@admin':'rw',
            '@super_reader':'r',
            '*':''
        }
        acl_parser[base_map_svn_directory] = {
            '@admin':'rw',
            '@super_reader':'r',
            '*':''
        }

def delete_task_file(project_name, base_file_directory):
    #FIXME  remove acl
    blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
    if svn_service.is_svn_url(blend_file_url):
        svn_service.svn_delete(blend_file_url, log_message=f'created {blend_file_url}')


import shutil
import os


def blend_file_gen(blend_file_path):
    dir_path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
    blender_template_file = os.path.join(dir_path,'template_files', 'blender.blend')
    # file_name = os.path.basename(blend_file_path)
    shutil.copy(blender_template_file, blend_file_path)

def task_directories(working_file_path):
    if working_file_path[0] == '/':
        file_directory = f'{working_file_path[1:]}'
        base_svn_directory = f"{file_directory.split('/', 2)[1]}:/{file_directory.split('/', 2)[2]}"
    else:
        file_directory = f'{working_file_path}'
        base_svn_directory = f"{file_directory.split('/', 3)[2]}:/{file_directory.split('/', 3)[3]}"
    return file_directory, base_svn_directory
def format_file_name(name):
    formated_name = name.replace(' ', '_').lower()
    return formated_name


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import (SVN_PARENT_PATH,
                                SVN_PARENT_URL,
                                LOGIN_NAME)
from genesys.app.blueprints.task.utils import create_task_file
from genesys.app.blueprints.project.routes import create_project
from genesys.app.services import svn_service, queue_store
from genesys.app.utils import config_helpers
import os
from configparser import ConfigParser
from genesys.app import config

refresh = Blueprint('refresh', __name__)
api = Api(refresh)

def refresh_project_files(data, project_name):
    project = (data['project'])
    root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'],'')
    # replacing file tree mount point with genesys config mount point
    base_file_directory = data['base_file_directory'].split(root,1)[1]
    project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
    if not svn_service.is_svn_url(project_repo_url):
        create_project(project_name)
    create_task_file(
                    task=data['task'],
                    project_name=project_name,
                    base_file_directory=base_file_directory,
                    base_svn_directory=data['base_svn_directory'])


def refresh_svn_acl(data, acl_parser):
    base_svn_directory = data['base_svn_directory']
    file_name = os.path.basename(base_svn_directory).rsplit('.', 1)[0]
    base_map_svn_directory = os.path.join(os.path.dirname(base_svn_directory), 'maps', file_name)
    permission = data['permission']

    all_users = list()
    for person in data['persons']:
        if person[LOGIN_NAME] != '':
            all_users.append(person[LOGIN_NAME])
    if all_users:
        for user in all_users:
            svn_service.svn_update_acl(
                base_svn_directory=base_svn_directory,
                acl_parser=acl_parser,
                person=user,
                permission=permission
            )
            svn_service.svn_update_acl(
                base_svn_directory=base_map_svn_directory,
                acl_parser=acl_parser,
                person=user,
                permission=permission
            )
            if data['permission'] == 'rw':
                for dependency in data['dependencies']:
                    dependency_file_name = os.path.basename(dependency).rsplit('.', 1)[0]
                    dependency_base_map_svn_directory = os.path.join(os.path.dirname(dependency), 'maps', dependency_file_name)
                    svn_service.svn_update_acl(
                        base_svn_directory=dependency,
                        acl_parser=acl_parser,
                        person=user,
                        permission='r'
                    )
                    svn_service.svn_update_acl(
                        base_svn_directory=dependency_base_map_svn_directory,
                        acl_parser=acl_parser,
                        person=user,
                        permission='r'
                    )
    else:
        if base_svn_directory not in acl_parser:
            acl_parser[base_svn_directory] = {
                '@admin':'rw',
                '@super_reader':'r',
                '*':''
            }
            acl_parser[base_map_svn_directory] = {
                '@admin':'rw',
                '@super_reader':'r',
                '*':''
            }

def refresh_project_data(all_data, project_name):
    new_acl_parser = ConfigParser()
    svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name, 'conf/authz')
    new_acl_parser['groups'] = {
                'admin':'super-user',
                'super_reader':'super-reader',
                'maps':'',
                'edit':'',
            }
    new_acl_parser['/'] = {
                '*':'r',
                '@admin':'rw',
                '@super_reader':'r',
            }
    for data in all_data:
        refresh_project_files(data, project_name)
    for data in all_data:
        refresh_svn_acl(data, new_acl_parser)

    config_helpers.write_config(svn_authz_path, new_acl_parser)




class All_Task_File_Access_Control(Resource):
    def post(self, project_name):
        data = request.get_json()
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                refresh_project_data,
                args=(data, project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            refresh_project_data(data, project_name)
            project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

api.add_resource(All_Task_File_Access_Control, '/refresh/<string:project_name>')


from flask import Blueprint, jsonify, request
from flask_restful import Api, Resource
from genesys.app.config import SVN_PARENT_PATH, SVN_PARENT_URL
from genesys.app.services import queue_store
import os
from genesys.app.utils import config_helpers
from genesys.app.blueprints.asset.utils import rename_asset_task_file, rename_task_acl_section
from configparser import ConfigParser
from genesys.app import config

asset = Blueprint('asset', __name__)
api = Api(asset)

def rename_asset(data, project_name):
    for asset_task in data:
        acl_parser = ConfigParser()
        task_type = asset_task['task_type']
        project = asset_task['project']
        root = os.path.join(project['file_tree']['working']['mountpoint'], project['file_tree']['working']['root'], '')
        # replacing file tree mount point with genesys config mount point
        base_file_directory = asset_task['base_file_directory'].split(root,1)[1]
        new_base_file_directory = asset_task['new_base_file_directory'].split(root,1)[1]
        base_svn_directory = asset_task['base_svn_directory']
        new_base_svn_directory = asset_task['new_base_svn_directory']

        blend_file_url = os.path.join(SVN_PARENT_URL, base_file_directory)
        new_blend_file_url = os.path.join(SVN_PARENT_URL, new_base_file_directory)

        svn_authz_path = os.path.join(SVN_PARENT_PATH, project_name.replace(' ', '_').lower(), 'conf/authz')

        config_helpers.load_config(svn_authz_path, acl_parser)
        rename_asset_task_file(
            old_blend_file_url=blend_file_url,
            new_blend_file_url=new_blend_file_url)
        rename_task_acl_section(
            old_base_svn_directory= base_svn_directory,
            base_svn_directory= new_base_svn_directory,
            acl_parser= acl_parser,
            )
        config_helpers.write_config(svn_authz_path, acl_parser)

class Asset(Resource):
    def put(self, project_name):
        data = request.get_json()
        project_repo_url = os.path.join(SVN_PARENT_URL, project_name)
        if config.ENABLE_JOB_QUEUE:
            queue_store.job_queue.enqueue(
                rename_asset,
                args=(data, project_name),
                job_timeout=10,
            )
            return {"job": "running"}
        else:
            rename_asset(data, project_name)
            return jsonify(message=f'project created', project_name=project_name, svn_url=project_repo_url)

api.add_resource(Asset, '/asset/<string:project_name>')




import os
from genesys.app.services import svn_service

def write_config(file_directory, config_parser):
    '''
        write data from a configuration parser
        to file
    '''
    with open(file_directory, 'w') as f:
        config_parser.write(f)

def load_config(file_directory, config_parser):
    '''
        load data from a configuration file 
        into a configuration parser instance
    '''
    with open(file_directory, 'r') as f:
        data = f.read()
    config_parser.read_string(data)


import os
SVN_PARENT_PATH = os.getenv("SVN_PARENT_PATH", '/opt/svn/repositories')
SVN_PARENT_URL = os.getenv("SVN_PARENT_URL", "file:////opt/svn/repositories")
LOGIN_NAME = os.getenv("LOGIN_NAME", "email")
TEMPLATE_FILES_DIR =os.path.join(os.path.dirname(__file__), 'template_files')

ENABLE_JOB_QUEUE = os.getenv("ENABLE_JOB_QUEUE", "False").lower() == "true"
ENABLE_JOB_QUEUE_REMOTE = (
    os.getenv("ENABLE_JOB_QUEUE_REMOTE", "False").lower() == "true"
)

KEY_VALUE_STORE = {
    "host": os.getenv("KV_HOST", "localhost"),
    "port": os.getenv("KV_PORT", "6379"),
}
KV_JOB_DB_INDEX = 7


from flask import Flask
from . import config
app = Flask('genesys')

from genesys.app.blueprints.index.routes import index
from genesys.app.blueprints.project.routes import project
from genesys.app.blueprints.task.routes import task
from genesys.app.blueprints.asset.routes import asset
from genesys.app.blueprints.refresh.routes import refresh


app.register_blueprint(index)
app.register_blueprint(project)
app.register_blueprint(task)
app.register_blueprint(asset)
app.register_blueprint(refresh)
app.config.from_object(config)



from flask import Blueprint
from flask_restful import Api, Resource
from genesys import __version__

index = Blueprint('index', __name__)
api = Api(index)

class Index(Resource):
    def get(self):
        return {"api": "eaxum_zou", "version": __version__}


api.add_resource(Index, '/')